@@ -10,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk
+from gi.repository import Gtk, GLib
 
 from random import shuffle
 
@@ -18,13 +18,15 @@ from lollypop.utils import get_human_duration, popup_widget
 from lollypop.utils_album import tracks_to_albums
 from lollypop.define import App, ArtSize, ViewType, Size
 from lollypop.widgets_banner import BannerWidget
+from lollypop.helper_signals import SignalsHelper, signals_map
 
 
-class PlaylistBannerWidget(BannerWidget):
+class PlaylistBannerWidget(BannerWidget, SignalsHelper):
     """
         Banner for playlist
     """
 
+    @signals_map
     def __init__(self, playlist_id, view):
         """
             Init banner
@@ -32,6 +34,7 @@ class PlaylistBannerWidget(BannerWidget):
             @param view as AlbumsListView
         """
         BannerWidget.__init__(self, view.args["view_type"] | ViewType.OVERLAY)
+        self.__duration_task = False
         self.__playlist_id = playlist_id
         self.__view = view
         builder = Gtk.Builder()
@@ -48,11 +51,14 @@ class PlaylistBannerWidget(BannerWidget):
         self._overlay.set_overlay_pass_through(widget, True)
         self.__title_label.set_label(App().playlists.get_name(playlist_id))
         builder.connect_signals(self)
-        # In DB duration calculation
-        if playlist_id > 0 and\
-                not App().playlists.get_smart(playlist_id):
-            duration = App().playlists.get_duration(playlist_id)
-            self.__duration_label.set_text(get_human_duration(duration))
+        self.connect("destroy", self.__on_destroy)
+        return [
+            (view, "initialized", "_on_view_updated"),
+            (App().player, "duration-changed", "_on_view_updated"),
+            (App().playlists, "playlist-track-added", "_on_playlist_changed"),
+            (App().playlists, "playlist-track-removed",
+             "_on_playlist_changed"),
+        ]
 
     def update_for_width(self, width):
         """
@@ -62,6 +68,13 @@ class PlaylistBannerWidget(BannerWidget):
         BannerWidget.update_for_width(self, width)
         self.__set_internal_size()
 
+    def rename(self, name):
+        """
+            Rename playlist
+            @param name as str
+        """
+        self.__title_label.set_label(name)
+
     @property
     def spinner(self):
         """
@@ -89,6 +102,17 @@ class PlaylistBannerWidget(BannerWidget):
         if BannerWidget._handle_width_allocate(self, allocation):
             self.__set_internal_size()
 
+    def _on_view_updated(self, *ignore):
+        """
+            Update duration
+        """
+        if self.__view.children:
+            self.__duration_task = True
+            App().task_helper.run(self.__calculate_duration)
+        else:
+            self.__duration_task = False
+            self.__duration_label.set_text(get_human_duration(0))
+
     def _on_play_button_clicked(self, button):
         """
             Play playlist
@@ -135,11 +159,33 @@ class PlaylistBannerWidget(BannerWidget):
         else:
             menu_widget = MenuBuilder(menu)
         menu_widget.show()
-        popup_widget(menu_widget, button)
+        popup_widget(menu_widget, button, None, None, button)
+
+    def _on_playlist_changed(self, playlists, playlist_id, uri):
+        """
+            Update duration
+            @param playlists as Playlists
+            @param playlist_id as int
+            @param uri as str
+        """
+        if self.__playlist_id == playlist_id:
+            self.__calculate_duration()
 
 #######################
 # PRIVATE             #
 #######################
+    def __calculate_duration(self):
+        """
+            Calculate playback duration
+        """
+        duration = 0
+        for child in self.__view.children:
+            if not self.__duration_task:
+                return
+            duration += child.album.duration
+        GLib.idle_add(self.__duration_label.set_text,
+                      get_human_duration(duration))
+
     def __set_internal_size(self):
         """
             Update font size
@@ -155,3 +201,17 @@ class PlaylistBannerWidget(BannerWidget):
         else:
             title_context.add_class("text-x-large")
             duration_context.add_class("text-large")
+
+    def __on_get_duration(self, duration):
+        """
+            Update label
+            @param duration as int
+        """
+        self.__duration_label.set_text(get_human_duration(duration))
+
+    def __on_destroy(self, widget):
+        """
+            Remove ref cycle
+            @param widget as Gtk.Widget
+        """
+        self.__view = None
